package com.kingdee.eas.custom.synctask;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.kingdee.bos.BOSException;
import com.kingdee.bos.Context;
import com.kingdee.bos.metadata.entity.EntityViewInfo;
import com.kingdee.bos.metadata.entity.FilterInfo;
import com.kingdee.bos.metadata.entity.FilterItemInfo;
import com.kingdee.bos.metadata.entity.SelectorItemCollection;
import com.kingdee.bos.metadata.query.util.CompareType;
import com.kingdee.eas.basedata.org.*;
import com.kingdee.eas.basedata.org.AdminOrgUnitCollection;
import com.kingdee.eas.basedata.org.AdminOrgUnitFactory;
import com.kingdee.eas.basedata.org.AdminOrgUnitInfo;
import com.kingdee.eas.basedata.org.IAdminOrgUnit;
import com.kingdee.eas.common.EASBizException;
import com.kingdee.eas.custom.log.ISyncLog;
import com.kingdee.eas.custom.log.SyncLogFactory;
import com.kingdee.eas.custom.log.SyncLogInfo;
import com.kingdee.eas.custom.log.app.DataDirectionEnum;
import com.kingdee.eas.custom.log.app.DockingSystemEnum;
import com.kingdee.eas.custom.log.app.SyncEntityNameEnum;
import com.kingdee.eas.custom.log.app.SyncStatusEnum;
import com.kingdee.eas.custom.synctask.entity.AdminOrg;
import com.kingdee.eas.custom.synctask.entity.PositionEntity;
import com.kingdee.eas.hr.ats.AtsUtil;
import com.kingdee.eas.util.app.DbUtil;
import com.kingdee.jdbc.rowset.IRowSet;
import com.kingdee.util.StringUtils;
import okhttp3.*;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.log4j.Logger;

import java.io.FileInputStream;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.sql.SQLException;
import java.util.*;


public class SyncTranForOAFacadeControllerBean extends AbstractSyncTranForOAFacadeControllerBean {
    private static Logger logger =
            Logger.getLogger(SyncTranForOAFacadeControllerBean.class);
    private Properties propt = new Properties();


    public SyncTranForOAFacadeControllerBean() throws BOSException {
        String syncOAConfigPath = System.getProperty("EAS_HOME") + "/server/properties/scy/syncOAConfig.properties";
        try {
            propt.load(new FileInputStream(syncOAConfigPath));
        } catch (IOException e) {
            e.printStackTrace();
            String errorMsg = "获取配置文件报错，请检查配置：" + syncOAConfigPath + "   " + e.getMessage();
            throw new BOSException(errorMsg);
        }
    }

    /**
     * 同步组织
     *
     * @param ctx
     * @param billds
     * @param action
     * @throws BOSException
     */
    @Override
    protected void _syncOrgUnitToOA(Context ctx, String billds, ActionTypeEnum action) throws BOSException, EASBizException {
        ISyncLog iSyncLog = SyncLogFactory.getLocalInstance(ctx);
        SyncLogInfo syncLogInfo = new SyncLogInfo();
        syncLogInfo.setEntityName(SyncEntityNameEnum.adminOrg);
        syncLogInfo.setDataDirection(DataDirectionEnum.outflow);
        syncLogInfo.setDockingSystem(DockingSystemEnum.OA);
        try {
            super._syncOrgUnitToOA(ctx, billds, action);
            logger.error("_syncOrgUnitToOA---start");
            logger.error("_syncOrgUnitToOA---billds: " + billds);
            IAdminOrgUnit iAdminOrgUnit = AdminOrgUnitFactory.getLocalInstance(ctx);
            List dataList = new ArrayList();
            //String actionType = action == null ? "" : action.getValue().toLowerCase();
            //当参数billIds不为空值时，增量同步参数中的部门数据到OA
            FilterInfo filterInfo = null;
            if (!StringUtils.isEmpty(billds)) {
                Set<String> adminOrgIds = AtsUtil.toSet(billds);
                filterInfo = new FilterInfo();
                filterInfo.getFilterItems().add(new FilterItemInfo("id", adminOrgIds, CompareType.INCLUDE));
            }
            SelectorItemCollection sic = new SelectorItemCollection();
            sic.add("*");
            sic.add("parent.number");
            EntityViewInfo entityViewInfo = EntityViewInfo.getInstance(filterInfo, sic, null);
            AdminOrgUnitCollection adminOrgUnitCollection = iAdminOrgUnit.getAdminOrgUnitCollection(entityViewInfo);
            for (int i = 0; i < adminOrgUnitCollection.size(); i++) {
                AdminOrgUnitInfo adminOrgUnitInfo = adminOrgUnitCollection.get(i);
                String orgName = adminOrgUnitInfo.getName();
                AdminOrg adminOrg = new AdminOrg();
                boolean isSealUp = adminOrgUnitInfo.isIsSealUp();
                if (isSealUp) {
                    adminOrg.setAction("delete");
                    adminOrg.setCanceled("0");
                } else {
                    adminOrg.setAction("add");
                }
                adminOrg.setCode(adminOrgUnitInfo.getNumber());
                adminOrg.setShortname(orgName);
                adminOrg.setFullname(orgName);
                //adminOrg.setParent_code("ITCS");
                adminOrg.setOrg_code("0");
                adminOrg.setParent_code(adminOrgUnitInfo.getParent().getNumber());
                //adminOrg.setOrder((String.valueOf(adminOrgUnitInfo.getIndex())));
                dataList.add(adminOrg);
            }
            //执行同步组织到OA
            executeSyncOrgUnitToOA(ctx, dataList, syncLogInfo);
        } catch (Exception e) {
            e.printStackTrace();
            syncLogInfo.setSyncStatus(SyncStatusEnum.ERROR);
            syncLogInfo.setSyncResult(e.getMessage());
            iSyncLog.save(syncLogInfo);
            throw new BOSException(e.getMessage());
        }
    }


    /**
     * 执行同步组织到OA
     *
     * @param dataList
     * @return
     */
    public void executeSyncOrgUnitToOA(Context ctx, List dataList, SyncLogInfo syncLogInfo) throws IOException, BOSException, EASBizException {
        if (dataList != null && dataList.size() > 0) {
            ObjectMapper mapper = new ObjectMapper();
            //封装请求参数
            String data = mapper.writeValueAsString(dataList);
            String params = String.format("token = %s &data = %s", getToken(), data);
            logger.error("executeSyncOrgUnitToOA----------------------params" + params);
            //封装请求参数
            String syncOrgUnitApiUrl = this.propt.getProperty("syncOrgUnitApiUrl");
            logger.error("executeSyncOrgUnitToOA----------------------syncOrgUnitApiUrl" + syncOrgUnitApiUrl);
            if (StringUtils.isEmpty(syncOrgUnitApiUrl)) {
                throw new BOSException("同步组织到OA接口地址不能为空!");
            }
            OkHttpClient client = new OkHttpClient();
            MediaType mediaType = MediaType.parse("application/x-www-form-urlencoded");
            RequestBody body = RequestBody.create(mediaType, params);
            Request request = new Request.Builder()
                    .url(syncOrgUnitApiUrl)
                    .post(body)
                    .addHeader("Content-Type", "application/json")
                    .addHeader("Accept", "*/*")
                    //.addHeader("Accept-Encoding", "gzip, deflate, br")
                    .addHeader("User-Agent", "PostmanRuntime-ApipostRuntime/1.1.0")
                    .addHeader("Connection", "keep-alive")
                    .build();
            Response response = client.newCall(request).execute();
            //处理响应结果,写入日志
            handlerResponse(ctx, response, syncLogInfo);
        }
    }

    /**
     * 处理响应结果,写入日志
     *
     * @param ctx
     * @param response
     * @param syncLogInfo
     * @throws IOException
     * @throws BOSException
     * @throws EASBizException
     */
    private void handlerResponse(Context ctx, Response response, SyncLogInfo syncLogInfo)
            throws IOException, BOSException, EASBizException {
        if (syncLogInfo == null) {
            return;
        }
        ISyncLog iSyncLog = SyncLogFactory.getLocalInstance(ctx);
        if (response != null) {
            int success = 0;
            int fail = 0;
            String respBody = response.body().string();
            JSONObject respMap = JSONObject.parseObject(respBody);
            logger.error("handlerResponse--------------respMap------" + respBody);
            if (response.isSuccessful()) {
                String code = respMap.getString("code");
                if ("1".equals(code)) {
                    StringBuilder msg = new StringBuilder();
                    for (String orgNumber : respMap.keySet()) {
                        JSONObject jsonObject = respMap.getJSONObject(orgNumber);
                        msg.append("[").append(orgNumber).append("]");
                        if ("1".equals(jsonObject.getString("code"))) {
                            success++;
                            msg.append("同步成功\n");
                        } else {
                            fail++;
                            msg.append("同步失败，").append(jsonObject.getString("msg")).append("\n");
                        }
                    }
                    syncLogInfo.setSyncStatus(SyncStatusEnum.SUCCESS);
                    syncLogInfo.setSyncCount(fail + success);
                    syncLogInfo.setSuccessNum(success);
                    syncLogInfo.setFailNum(fail);
                    syncLogInfo.setSyncResult(msg.toString());
                } else {
                    syncLogInfo.setSyncStatus(SyncStatusEnum.FAIL);
                    syncLogInfo.setSyncResult(respMap.getString("msg"));
                }
            } else {
                syncLogInfo.setSyncStatus(SyncStatusEnum.FAIL);
                syncLogInfo.setSyncResult("请求超时");
            }
        } else {
            syncLogInfo.setSyncStatus(SyncStatusEnum.FAIL);
            syncLogInfo.setSyncResult("同步数据为空");
        }
        iSyncLog.save(syncLogInfo);
    }


    /**
     * 获取token
     *
     * @return
     */
    private String getToken() throws BOSException {
        String key = this.propt.getProperty("key");
        if (StringUtils.isEmpty(key)) {
            throw new BOSException("获取token key不能为空！");
        }
        long l = System.currentTimeMillis();//时间戳毫秒数
        String code = key.concat(Long.toString(l));
        String s = DigestUtils.md5Hex(code).toUpperCase();
        Map<String, String> map = new HashMap<String, String>();
        map.put("key", s);
        map.put("ts", Long.toString(l));
        logger.error("getToken" + JSONObject.toJSONString(map));
        return JSONObject.toJSONString(map);
    }

    /**
     * 同步职位信息
     *
     * @param ctx
     * @param billds
     * @param action
     * @throws BOSException
     */
    @Override
    protected void _syncPositionToOA(Context ctx, String billds, ActionTypeEnum action) throws BOSException, EASBizException {
        ISyncLog iSyncLog = SyncLogFactory.getLocalInstance(ctx);
        SyncLogInfo syncLogInfo = new SyncLogInfo();
        syncLogInfo.setEntityName(SyncEntityNameEnum.postiton);
        syncLogInfo.setDataDirection(DataDirectionEnum.outflow);
        syncLogInfo.setDockingSystem(DockingSystemEnum.OA);
        try {
            super._syncPositionToOA(ctx, billds, action);
            logger.error("_syncPositionToOA---billds: " + billds);
            String data = getPositionColData(ctx, billds);
            String syncPositionApiUrl = this.propt.getProperty("syncPositionApiUrl");
            synScyRequest(ctx,data, syncPositionApiUrl,syncLogInfo);
        } catch (Exception e) {
            e.printStackTrace();
            syncLogInfo.setSyncStatus(SyncStatusEnum.ERROR);
            syncLogInfo.setSyncResult(e.getMessage());
            iSyncLog.save(syncLogInfo);
            throw new BOSException(e.getMessage());
        }


    }

    /**
     * 获取岗位信息
     * @param ctx
     * @param billIds
     * @return
     * @throws IOException
     * @throws BOSException
     */
    protected String getPositionColData(Context ctx,String billIds) throws IOException, BOSException {
        List list = new ArrayList();
        ObjectMapper mapper = new ObjectMapper();
        Set positionIds = new HashSet();
        PositionCollection positionCollection = new PositionCollection();
        IPosition localInstance = PositionFactory.getLocalInstance(ctx);
        if (org.apache.commons.lang.StringUtils.isNotBlank(billIds)){
            String[] split = billIds.split(",");
            for (int i = 0; i < split.length; i++) {
                positionIds.add(split[i]);
            }
            positionCollection = localInstance.getPositionCollection("where id in( " + AtsUtil.convertSetToString(positionIds) + ")");
        }else {
            positionCollection = localInstance.getPositionCollection();
        }
        for (int i = 0; i < positionCollection.size(); i++) {
            PositionEntity positionEntity = new PositionEntity();
            PositionInfo positionInfo = positionCollection.get(i);
            DeletedStatusEnum deletedStatus = positionInfo.getDeletedStatus();
            if ("2".equals(deletedStatus.getValue())){ // 失效
                positionEntity.setAction("delete");
            }else {
                positionEntity.setAction("add");
            }
            positionEntity.setJobtitlecode(positionInfo.getNumber());
            positionEntity.setJobtitlename(positionInfo.getName());
            positionEntity.setJobtitleremark(positionInfo.getName());
            list.add(positionEntity);
        }
        //封装请求参数
        String data = mapper.writeValueAsString(list);
        logger.error("_syncPositionToOA----------------------data" + data);
        return data;
    }

    /**
     * 同步请求方法
     * @param param
     * @param aipUrl
     * @return
     * @throws IOException
     */
    public void synScyRequest(Context ctx,String param, String aipUrl, SyncLogInfo syncLogInfo) throws IOException, BOSException, EASBizException {
        OkHttpClient client = new OkHttpClient();
        MediaType mediaType = MediaType.parse("application/x-www-form-urlencoded");
        String data = String.format("token = %s &data = %s", getToken(), param);
        //logger.error("synAdminOrgRequest----------------------date" + data);
        RequestBody body = RequestBody.create(mediaType, data);
        Request request = new Request.Builder()
                .url(aipUrl)
                .post(body)
                .addHeader("Content-Type", "application/json")
                .addHeader("Accept", "*/*")
                //.addHeader("Accept-Encoding", "gzip, deflate, br")
                .addHeader("User-Agent", "PostmanRuntime-ApipostRuntime/1.1.0")
                .addHeader("Connection", "keep-alive")
                .build();
        //处理响应结果,写入日志
        Response response = client.newCall(request).execute();
        handlerResponse(ctx, response, syncLogInfo);
    }


    /**
     * 同步员工信息
     *
     * @param ctx
     * @param billds
     * @param action
     * @throws BOSException
     */
    @Override
    protected void _syncPersonToOA(Context ctx, String billds, ActionTypeEnum action) throws BOSException, EASBizException {
        super._syncPersonToOA(ctx, billds, action);
    }

    /**
     * 根据id从OA系统同步人员信息
     *
     * @param ctx
     * @param personIds
     * @throws BOSException
     * @throws EASBizException
     */
    @Override
    protected void _syncPersonFromOAById(Context ctx, String personIds) throws BOSException, EASBizException {
        super._syncPersonFromOAById(ctx, personIds);
        if (StringUtils.isEmpty(personIds)) {
            this._syncPersonFromOAByNumber(ctx, null);
        } else {
            try {
                String query = "select fnumber from t_bd_person where fid in (" + AtsUtil.getStrFromString(personIds) + ")";
                IRowSet iRowSet = DbUtil.executeQuery(ctx, query);
                StringBuilder personNumbers = new StringBuilder();
                while (iRowSet.next()) {
                    String fnumber = iRowSet.getString("fnumber");
                    personNumbers.append(fnumber).append(",");
                }
                personNumbers.deleteCharAt(personNumbers.lastIndexOf(","));
                this._syncPersonFromOAByNumber(ctx, personNumbers.toString());
            } catch (SQLException e) {
                e.printStackTrace();
                throw new BOSException("根据id从OA系统同步人员信息方法报错: " + e.getMessage());
            }
        }
    }

    /**
     * 根据工号从OA系统同步人员信息
     *
     * @param ctx
     * @param personNumbers
     * @throws BOSException
     * @throws EASBizException
     */
    @Override
    protected void _syncPersonFromOAByNumber(Context ctx, String personNumbers) throws BOSException, EASBizException {
        super._syncPersonFromOAByNumber(ctx, personNumbers);
        ISyncLog iSyncLog = SyncLogFactory.getLocalInstance(ctx);
        SyncLogInfo syncLogInfo = new SyncLogInfo();
        syncLogInfo.setEntityName(SyncEntityNameEnum.person);
        syncLogInfo.setDataDirection(DataDirectionEnum.flowInto);
        syncLogInfo.setDockingSystem(DockingSystemEnum.OA);
        logger.error("_syncPersonFromOAByNumber -> personNumbers" + personNumbers);
        try {
            String updatePersonSql = "update t_bd_person set cfOAId = ? where fnumber = ?";
            int pagesize = 2000;//每页数量
            int curpage = 1;//当前页
            int totalSize = 0;//总条数
            int totalPage = 0;//总页数
            List<Object[]> updateParams = new ArrayList<Object[]>();
            boolean flag = true;
            do {
                totalSize = executeSyncPersonFromOA(updateParams, curpage, pagesize, personNumbers);
                if (flag) {
                    totalPage = totalSize % pagesize == 0 ? totalSize / pagesize : totalSize / pagesize + 1;
                    flag = false;
                }
                curpage++;
            } while (curpage <= totalPage);
            if (!updateParams.isEmpty()) {
                DbUtil.executeBatch(ctx, updatePersonSql, updateParams);
            }
            syncLogInfo.setSyncStatus(SyncStatusEnum.SUCCESS);
            syncLogInfo.setSyncCount(updateParams.size());
            syncLogInfo.setSuccessNum(updateParams.size());
            iSyncLog.save(syncLogInfo);
        } catch (IOException e) {
            e.printStackTrace();
            syncLogInfo.setSyncStatus(SyncStatusEnum.ERROR);
            syncLogInfo.setSyncResult(e.getMessage());
            iSyncLog.save(syncLogInfo);
            throw new BOSException(e.getMessage());
        }
    }


    /**
     * 执行从OA系统同步人员信息
     *
     * @param updateParams
     * @param curpage
     * @return
     */
    public Integer executeSyncPersonFromOA(List<Object[]> updateParams, int curpage, int pagesize, String personNumbers)
            throws IOException, BOSException, EASBizException {
        //封装请求参数
        String params = null;
        if (StringUtils.isEmpty(personNumbers)) {
            params = String.format("token = %s &params = {\"pagesize\":%d,\"curpage\":%d}", getToken(), pagesize, curpage);
        } else {
            params = String.format("token = %s &params = {\"workcode\":\"%s\",\"pagesize\":%d,\"curpage\":%d}", getToken(), personNumbers, pagesize, curpage);
        }
        logger.error("executeSyncPersonFromOA----------------------params" + params);
        //封装请求参数
        String syncPersonFromApiUrl = this.propt.getProperty("syncOrgUnitApiUrl");
        logger.error("executeSyncPersonFromOA----------------------syncOrgUnitApiUrl" + syncPersonFromApiUrl);
        if (StringUtils.isEmpty(syncPersonFromApiUrl)) {
            throw new BOSException("获取人员接口地址不能为空!");
        }
        OkHttpClient client = new OkHttpClient();
        MediaType mediaType = MediaType.parse("application/x-www-form-urlencoded");
        RequestBody body = RequestBody.create(mediaType, params);
        Request request = new Request.Builder()
                .url(syncPersonFromApiUrl)
                .post(body)
                .addHeader("content-type", "application/x-www-form-urlencoded")
                .build();
        Response response = client.newCall(request).execute();
        String respBody = response.body().string();
        JSONObject respMap = JSONObject.parseObject(respBody);
        logger.error("handlerResponse--------------respMap" + respBody);
        if (response.isSuccessful()) {
            String code = respMap.getString("code");
            String msg = respMap.getString("msg");
            if ("1".equals(code)) {
                JSONObject respData = respMap.getJSONObject("data");
                Integer totalSize = respData.getInteger("totalSize");
                JSONArray respDataList = respData.getJSONArray("dataList");
                for (int i = 0; i < respDataList.size(); i++) {
                    JSONObject jsonObject = respDataList.getJSONObject(i);
                    String workcode = jsonObject.getString("workcode");
                    if (!StringUtils.isEmpty(workcode)) {
                        Object[] objects = new Object[2];
                        objects[0] = jsonObject.getString("id");
                        objects[1] = workcode;
                        updateParams.add(objects);
                    }
                }
                return totalSize;
            } else {
                throw new BOSException(msg);
            }
        } else {
            throw new BOSException("请求超时");
        }
    }
}
